/*
Copyright 2011 Philip Schweiger <pschwei1@gmail.com>

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

/**
 * @class FSW - wrapper for the Facebook JavaScript SDK.
 * (not endorsed by or affiliated with Facebook).
 * @version 0.1.0
 * @author Philip Schweiger [pschwei1@gmail.com]
 */
var fsw = (function ($) {

    /**
     * Store basic information about the application
     * @ignore
     */
	var _appId,_canvasUrl,_tabUrl;
   
    return {
		/**
		* jQuery deffered object; resolved once FB JS SDK has loaded
		* @ignore
		*/
        fbLoaded: $.Deferred(),

		/**
		 * Calls FB.init() and stores basic app info for later retrieval.
		 * Also adds a namespace to [html] tag to support rendering xfbml.
		 *
		 * @memberOf fsw
		 * @param {String} appId application ID
		 * @param {String} [canvasUrl] Fully-qualified app canvas URL
		 * @param {String} [tabUrl] Fully-qualified app tab URL
		 */
        init:function(appId, canvasUrl, tabUrl){
            if (typeof appId === 'undefined') {
                throw new Error('fsw.init(): Missing appId');
            }
		
            _appId = appId;
            _canvasUrl = (typeof canvasUrl === 'undefined')? null : canvasUrl;
            _tabUrl = (typeof tabUrl === 'undefined')? null : tabUrl;
		
            $('html').attr('xmlns:fb','http://www.facebook.com/2008/fbml');

            FB.init({appId:_appId,status:true,cookie:true,xfbml:true,oauth:true});
		},
        
		/**
		* Getter
		* @memberOf fsw
		* @returns {String} application id
		*/
        getAppId: function(){
            return _appId;
        },

		/**
		* Getter
		* @memberOf fsw
		* @returns {String} canvas url
		*/
        getCanvasUrl: function(){
            return _canvasUrl;
        },

		/**
		* Getter
		* @memberOf fsw
		* @returns {String} tab url
		*/
        getTabUrl: function(){
            return _tabUrl;
        },

		/**
		* @class Authorization module. Handles login and permissions.
		* @name fsw.auth
		*/   
		auth: {
			/**
			* Bind FB.login() to a specified jQuery DOM element
			* 
			* @memberOf fsw.auth
			* @param {Object} trigger jQuery DOM element
			* @param {Function} callback function to launch after login
			* @param {String} [permissions] Comma separated list, 
			* see http://bit.ly/hOUAQD and http://bit.ly/cjEOKB
			*/
			createBtn: function(trigger,callback, permissions){
				if (!(trigger instanceof jQuery)) {
					throw new Error('fsw.auth.createBtn: must pass a jquery DOM element as trigger');
				}

				if (typeof callback !== 'function') {
					throw new Error('fsw.auth.createBtn: must pass a callback function');
				}
			
				var perms = (typeof permissions !== 'undefined')? {perms:permissions}: null;

				trigger.click(function(e){
				   e.preventDefault(); 
				   FB.login(callback,perms);
				});

			}
		},
   
		/**
		* @class Like module. Handles the like button and related events.
		* @name fsw.like
		*/
		like: {
			/**
			* Constructor for likeBtn instances
			* @memberOf fsw.like
			* @private
			* @returns {LikeBtn}
			*/
			likeBtn: function($container, target, opts, layoutOpts){
				var action = (opts.action === 'recommend')? 'recommend' : 'like', 
					colorscheme = (opts.colorscheme === 'dark')? 'dark' : 'light',
					faces = opts.faces === true,
					font = opts.font || 'arial',
					layout = ($.inArray(opts.layout,layoutOpts) > -1)? opts.layout: 'standard',
					ref = (typeof opts.ref === 'string')? 'ref="'+opts.ref+'"': '',
					send = opts.send === true,
					width = (parseInt(opts.width,10) > 0)? parseInt(opts.width,10) : 450;

				   $container.append('<fb:like href="'+target+'" action="'+action+'" send="'+send+'" width="'+width+'" show_faces="'+faces+'" font="'+font+'" layout="'+layout+'" colorscheme="'+colorscheme+'" '+ref+'></fb:like>');
					FB.XFBML.parse();
			  
				return this;
			},
		
			/**
			* Create a FB "like" button
			* @memberOf fsw.like
			* @param {Object} $container jQuery DOM element to insert the Like button into
			* @param {String} target URL to Like
			* @param [options] Additional information passed to the like button
			* @param {String} options.action Default: like - Alts: recommend
			* @param {String} options.colorscheme Default: light - Alts: dark
			* @param {Boolean} options.faces Default: false - Show fans' profile pics
			* @param {String} options.font Default: arial - Alts: lucida grande | segoe ui | tahoma | trebuchet ms | verdana
			* @param {String} options.layout Default: standard - Alts: button_count | box_count
			* @param {String} options.ref a label for tracking referrals
			* @param {Boolean} options.send Default: false - Include a "send" button?
			* @param {Number} options.width Default: 420 - Width, in pixels, of plugin
			*/
			createBtn: function($container, target, options) {
				var err = new Error(),
					opts = options || {},
					layoutOpts = ['standard','button_count','box_count'];

				if ($container === undefined) {
				   err.message = 'fws.like.createBtn(): Missing container';
				   throw err;
				}

				if (!($container instanceof jQuery)) {
				   err.message = 'fws.like.createBtn(): Container is not a jQuery object';
				   throw err;
				}

				if (target === undefined) {
				   err.message = 'fws.like.createBtn(): Missing "like" target';
				   throw err;
				}
			   return new this.likeBtn($container, target, opts, layoutOpts);
			}
		},
   
		/**
		* @class Send module. Handles the send button and related events.
		* @name fsw.send
		*/
		send: {
			/**
			* Constructor for sendBtn instances
			* @memberOf fsw.send
			* @private
			* @returns {SendBtn}
			*/
			sendBtn: function($container, target, opts){

				var colorscheme = (opts.colorscheme === 'dark')? 'dark' : 'light',
					font = opts.font || 'arial',
					ref = (typeof opts.ref === 'string')? 'ref="'+opts.ref+'"': '';

				   $container.append('<fb:send href="'+target+'" font="'+font+'" colorscheme="'+colorscheme+'" '+ref+'></fb:send>');
					FB.XFBML.parse();
			 
				return this;
			},    
		
			/**
			* Create a FB "send" button
			* @memberOf fsw.send
			* @param {Object} $container jQuery DOM element to insert the Like button into
			* @param {String} target URL to Like
			* @param [options] Additional information passed to the like button
			* @param {String} options.colorscheme Default: light - Alts: dark
			* @param {String} options.font Default: arial - Alts: lucida grande | segoe ui | tahoma | trebuchet ms | verdana
			* @param {String} options.ref a label for tracking referrals
			*/
			createBtn: function($container, target, options) {
				var err = new Error(),
					opts = options || {};

				if ($container === undefined) {
				   err.message = 'fsw.send.createBtn(): Missing container';
				   throw err;
				}

				if (!($container instanceof jQuery)) {
				   err.message = 'fsw.send.createBtn(): Container is not a jQuery object';
				   throw err;
				}

				if (target === undefined) {
				   err.message = 'fsw.send.createBtn(): Missing "send" target';
				   throw err;
				}

				return new this.sendBtn($container, target, opts);
			}
		},

		/**
		* @class Stream module. Handles stream interactions, such as posting to Wall.
		* @name fsw.stream
		*/
		stream: {
			/**
			* Post to current user's wall/stream
			* @memberOf fsw.stream
			* @public
			* @param {Object} [opts] Options for customizing the wall post
			* @param {String} [opts.title] Title of the post. Defaults to blank.
			* @param {String} [opts.subtitle] Subtitle of the post
			* @param {String} [opts.link] The link attached to this post. 
			* Must be within your app's domain.
			* @param {String} [opts.description] App-supplied text to appear in the post. 
			* Defaults to blank. If no link and no media, description will not appear.
			* @param {String} [opts.media] URL of media to display with the post. FB 
			* will try to pull an image from linked page if this is missing. If you pass a swf, you must also pass a thumbnail image in comma-separated format, eg "http://domain/movie.swf, http://domain/image.jpg"
			* @param {String} [opts.action] Comma-separated pair of verb and link, 
			* eg "action,http://www.example.com" Link must be within your app's domain, 
			* and name becomes lowercase alphanumeric.
			* @param {Function} [callback] Function to call after user sends or cancels 
			* post. Returns null on user cancel, post id on user send.
			*/    
			post: function(options, callback){
				var opts = options || {},
					name = opts.title || ' ',
					cap = opts.subtitle || ' ',
					link = opts.link || null,
					source = opts.media || null,
					desc = opts.description || ' ',
					action = (function(){
					  if (opts.action) {
						  var action = opts.action.split(',');
						  return {name:action[0],link:$.trim(action[1])};
					  } else {
						  return null;
					  }
					}()),
					cb = (typeof callback === 'function')? callback : null,
					
					fbparams = {
						actions:action,
						caption:cap,
						description:desc,
						link:link,
						method:'feed',
						name:name,
						source:source
					};
				//If this is a swf, set a thumbnail
				if (source && source.indexOf('.swf') > -1) {
				
					if (source.indexOf(',') === -1) {
						throw new Error('fsw.stream.post(): Missing thumbnail for swf');
					} else {
						var parts = source.split(',');
						fbparams.source = parts[0];
						fbparams.picture = $.trim(parts[1]);
					}
				}
				
				FB.ui(fbparams,cb);
			}  
		},

		/**
		* @class Request module. Sends app requests.
		* @name fsw.request
		*/
		request: {
			/**
			*Send an app request to a specific friend, or to several friends. Note that this module does not address server-side handling of app requests.
			*@memberOf fsw.request
			*@param {Object} opts
			*@param {String} opts.message Request message
			*@param {String} [opts.to] A FBID or FB user name to send the request to. Must be a friend of the user.
			*@param {String} [opts.data] Data to send along with the request. Better explanation TK.
			*@param {Function}[callback] Function to call after user sends or cancels post. Returns null on user cancel, request id on user send.
			*/
			send: function(opts, callback){
				if (typeof opts.message !== 'string') {
					throw new Error('Must pass opts.message as a string');
				}
					
				var data = opts.data || null,
					to = opts.to || null;
					
				callback = (typeof callback === 'function')? callback : null;
					
				FB.ui(
					{
						data:data,
						message:opts.message,
						method:'apprequests',
						to:to
					},
					callback
				);
			}  
		}
	}
    
}(jQuery));

/**
* Bind to "like" buttons' like/unlike events
* @memberOf fsw.like
* @param {Function} onLike callback when user "likes" using FB like button
* @param {Function} onUnlike callback when user "unlikes" using FB like button
* 
* @example var myButton = fsw.like.createBtn($container,target); 
* myButton.bind(onLike, onUnlike);
*/
fsw.like.likeBtn.prototype.bind = function(onLike,onUnlike){
        
	if (typeof onLike !== 'function' || typeof onUnlike !== 'function') {
		throw new Error("must pass an onLike and an onUnlike callback as function");
	}
	
	FB.Event.subscribe('edge.create',onLike);
	FB.Event.subscribe('edge.remove',onUnlike);
	
};

/**
* Bind to "like" buttons' like/unlike events
* @memberOf fsw.send
* @param {Function} onSend callback when user "sends" using FB send button
* 
* @example var myButton = fsw.send.createBtn($container,target); 
* myButton.bind(onSend);
*/
fsw.send.sendBtn.prototype.bind = function(onSend){
        
	if (typeof onSend !== 'function') {
		throw new Error("must pass an onSend callback as function");
	}

	FB.Event.subscribe('message.send',onSend);
};

/**
 * On load, create and insert #fb-root into the DOM. Don't wait for init();
 * @ignore
 */
$(function(){
    $('#fb-root').remove();//in case one is already there
    $('body').append('<div id="fb-root"></div>');
});

/**
 * Load the FB JS SDK, and resolve the fbLoaded deferred object when done;
 * @ignore
 */
$.getScript('https://connect.facebook.net/en_US/all.js',function(){
	fsw.fbLoaded.resolve();
});